@@ -26,7 +26,7 @@ Date::Format - Date formating subroutines
 	print ascctime(@lt);
 	
 	print ctime(time, $zone);
-	print ascctime(@lt, $zone);
+	print asctime(@lt, $zone);
 
 =head1 DESCRIPTION
 
@@ -59,6 +59,25 @@ conversion specification C<"%a %b %e %T %Y\n">
 
 =back
 
+=head1 MULTI-LANGUAGE SUPPORT
+
+Date::Format is capable of formating into several languages, these are
+English, French, German and Italian. Changing the language is done via
+a static method call, for example
+
+	Date::Format->language('German');
+
+will change the language in which all subsequent dates are formatted.
+
+This is only a first pass, I am considering changing this to be
+
+	$lang = Date::Language->new('German');
+	$lang->time2str("%a %b %e %T %Y\n", time);
+
+I am open to suggestions on this.
+
+=head1 CONVERSION SPECIFICATION
+
 Each conversion specification  is  replaced  by  appropriate
 characters   as   described  in  the  following  list.   The
 appropriate  characters  are  determined  by   the   LC_TIME
@@ -98,6 +117,7 @@ category of the program's locale.
 	%y	year (2 digits)
 	%Y	year (4 digits)
 	%Z 	timezone in ascii. eg: PST
+	%z	timezone in format -/+0000
 
 =head1 AUTHOR
 
@@ -105,7 +125,7 @@ Graham Barr <Graham.Barr@tiuk.ti.com>
 
 =head1 REVISION
 
-$Revision: 2.3 $
+$Revision: 2.5 $
 
 =head1 COPYRIGHT
 
@@ -115,125 +135,100 @@ as Perl itself.
 
 =cut
 
-use Time::Zone;
-use Time::Local;
-
-use strict;
-no strict qw(refs);
-
-use vars qw(@DoW @MoY @DoWs @MoYs @AMPM @Dsuf %locale);
-use vars qw($sec $min $hour $mday $mon $year $wday $yday $isdst $tzname $epoch);
-use vars qw(@EXPORT @ISA $VERSION);
-
-$VERSION = sprintf("%d.%02d", q$Revision: 2.3 $ =~ /(\d+)\.(\d+)/);
-
+use     strict;
+use     vars qw(@EXPORT @ISA $VERSION);
 require Exporter;
-@ISA = qw(Exporter);
-@EXPORT = qw(time2str strftime ctime asctime);
 
-BEGIN {
-
- @DoW = qw(Sunday Monday Tuesday Wednesday Thursday Friday Saturday);
-
- @MoY = qw(January February March April May June
-	   July August September October November December);
-
- @DoWs = map { substr($_,0,3) } @DoW;
- @MoYs = map { substr($_,0,3) } @MoY;
- 
- @AMPM = qw(AM PM);
-
- @Dsuf = (qw(th st nd rd th th th th th th)) x 3;
- @Dsuf[11,12,13] = qw(th th th);
-
- %locale = (	'x' => "%m/%d/%y",
-		'C' => "%a %b %e %T %Z %Y",
-		'X' => "%H:%M:%S",
-	     );
-
- my @locale;
- my $locale = "/usr/share/lib/locale/LC_TIME/default";
- local *LOCALE;
-
- if(open(LOCALE,"$locale")) {
-  chop(@locale = <LOCALE>);
-  close(LOCALE);
-
-  @MoYs = @locale[0 .. 11];
-  @MoY  = @locale[12 .. 23];
-  @DoWs = @locale[24 .. 30];
-  @DoW  = @locale[31 .. 37];
-  @locale{"X","x","C"} =  @locale[38 .. 40];
-  @AMPM = @locale[41 .. 42];
- }
-}
-
-#
-# Taken from Time::ParseDate by David Muir Sharnoff <muir@idiom.com>
-#
-
-sub wkyr {
-    my($wstart, $wday, $yday) = @_;
-    $wday = ($wday + 7 - $wstart) % 7;
-    return int(($yday - $wday + 13) / 7 - 1);
-}
-
-sub a { $DoWs[$wday] }
-sub A { $DoW[$wday] }
-sub b { $MoYs[$mon] }
-sub B { $MoY[$mon] }
-sub d { sprintf("%02d",$mday) }
-sub e { sprintf("%2d",$mday) }
-sub H { sprintf("%02d",$hour) }
-sub h { $MoYs[$mon] }
-sub I { sprintf("%02d",$hour % 12 || 12)}
-sub j { sprintf("%03d",$yday + 1) }
-sub k { sprintf("%2d",$hour) }
-sub l { sprintf("%2d",$hour % 12 || 12)}
-sub m { sprintf("%02d",$mon + 1) }
-sub M { sprintf("%02d",$min) }
-sub p { $hour >= 12 ?  $AMPM[1] : $AMPM[0] }
-sub s { sprintf("%d",$epoch) }
-sub S { sprintf("%02d",$sec) }
-sub U { wkyr(0, $wday, $yday) }
-sub w { $wday }
-sub W { wkyr(1, $wday, $yday) }
-sub y { sprintf("%02d",$year % 100) }
-sub Y { sprintf("%04d",$year + 1900) }
-sub Z { defined $tzname ? $tzname : uc tz_name(undef, $isdst); }
-
-sub c { &x . " " . &X }
-sub D { &m . "/" . &d . "/" . &y  }      
-sub r { &I . ":" . &M . ":" . &S . " " . &p  }   
-sub R { &H . ":" . &M }
-sub T { &H . ":" . &M . ":" . &S }
-sub t { "\t" }
-sub n { "\n" }
-
-sub o { sprintf("%2d%s",$mday,$Dsuf[$mday]) }
-
-sub locale_fmt
-{
- my $ch = shift;
- my $fmt = $locale{$ch};
-
- $fmt =~ s#%([%a-zA-Z])#expand($1)#sge;
-
- $fmt;
-}
-
-sub x { locale_fmt('x'); }
-sub X { locale_fmt('X'); }
-sub C { locale_fmt('C'); }
-
-
-sub expand { defined &{$_[0]} ? &{$_[0]} : $_[0]; }
+$VERSION = sprintf("%d.%02d", q$Revision: 2.5 $ =~ /(\d+)\.(\d+)/);
+@ISA     = qw(Exporter);
+@EXPORT  = qw(time2str strftime ctime asctime);
 
 sub time2str ($;$$)
 {
- my($fmt,$time);
+ Date::Format::Generic->time2str(@_);
+}
 
- ($fmt,$time,$tzname) = @_;
+sub strftime ($\@;$)
+{
+ Date::Format::Generic->strftime(@_);
+}
+
+sub ctime ($;$)
+{
+ my($t,$tz) = @_;
+ Date::Format::Generic->time2str("%a %b %e %T %Y\n", $t, $tz); 
+}
+
+sub asctime (\@;$)
+{
+ my($t,$tz) = @_;
+ Date::Format::Generic->strftime("%a %b %e %T %Y\n", $t, $tz); 
+}
+
+##
+##
+##
+
+package Date::Format::Generic;
+
+use vars qw($epoch $tzname);
+use Time::Zone;
+use Time::Local;
+
+sub ctime
+{
+ my($me,$t,$tz) = @_;
+ $me->time2str("%a %b %e %T %Y\n", $t, $tz); 
+}
+
+sub asctime
+{
+ my($me,$t,$tz) = @_;
+ $me->strftime("%a %b %e %T %Y\n", $t, $tz); 
+}
+
+sub _subs
+{
+ $_[1] =~ s,%([%a-zA-Z]), $1 eq '%' ? '%' : $_[0]->$1() ,sgeo;
+
+ $_[1];
+}
+
+sub strftime 
+{
+ my($pkg,$fmt,$time);
+
+ ($pkg,$fmt,$time,$tzname) = @_;
+
+ my $me = ref($pkg) ? $pkg : bless [];
+
+ if(defined $tzname)
+  {
+   $tzname = uc $tzname;
+
+   $tzname = sprintf("%+05d",$tzname)
+	unless($tzname =~ /\D/);
+
+   $epoch = timegm(@{$time}->[0..5]);
+
+   @$me = gmtime($epoch + tz_offset($tzname) - tz_offset());
+  }
+ else
+  {
+   @$me = @$time;
+   undef $epoch;
+  }
+
+ _subs($me,$fmt);
+}
+
+sub time2str
+{
+ my($pkg,$fmt,$time);
+
+ ($pkg,$fmt,$time,$tzname) = @_;
+
+ my $me = ref($pkg) ? $pkg : bless [], $pkg;
 
  $epoch = $time;
 
@@ -245,58 +240,123 @@ sub time2str ($;$$)
 	unless($tzname =~ /\D/);
 
    $time += tz_offset($tzname);
-   ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = gmtime($time);
+   @$me = gmtime($time);
   }
  else
   {
-   ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime($time);
+   @$me = localtime($time);
   }
-
- $fmt =~ s#%([%a-zA-Z])#expand($1)#sge;
-
- $fmt;
+ _subs($me,$fmt);
 }
 
-sub strftime ($\@;$)
+my(@DoW,@MoY,@DoWs,@MoYs,@AMPM,%format,@Dsuf);
+
+@DoW = qw(Sunday Monday Tuesday Wednesday Thursday Friday Saturday);
+
+@MoY = qw(January February March April May June
+          July August September October November December);
+
+@DoWs = map { substr($_,0,3) } @DoW;
+@MoYs = map { substr($_,0,3) } @MoY;
+
+@AMPM = qw(AM PM);
+
+@Dsuf = (qw(th st nd rd th th th th th th)) x 3;
+@Dsuf[11,12,13] = qw(th th th);
+
+%format = ('x' => "%m/%d/%y",
+           'C' => "%a %b %e %T %Z %Y",
+           'X' => "%H:%M:%S",
+          );
+
+my @locale;
+my $locale = "/usr/share/lib/locale/LC_TIME/default";
+local *LOCALE;
+
+if(open(LOCALE,"$locale"))
+ {
+  chop(@locale = <LOCALE>);
+  close(LOCALE);
+
+  @MoYs = @locale[0 .. 11];
+  @MoY  = @locale[12 .. 23];
+  @DoWs = @locale[24 .. 30];
+  @DoW  = @locale[31 .. 37];
+  @format{"X","x","C"} =  @locale[38 .. 40];
+  @AMPM = @locale[41 .. 42];
+ }
+
+sub wkyr {
+    my($wstart, $wday, $yday) = @_;
+    $wday = ($wday + 7 - $wstart) % 7;
+    return int(($yday - $wday + 13) / 7 - 1);
+}
+
+##
+## these 6 formatting routins need to be *copied* into the language
+## specific packages
+##
+
+sub a { $DoWs[$_[0]->[6]] }
+sub A { $DoW[$_[0]->[6]] }
+sub b { $MoYs[$_[0]->[4]] }
+sub B { $MoY[$_[0]->[4]] }
+sub h { $MoYs[$_[0]->[4]] }
+sub p { $_[0]->[2] >= 12 ?  $AMPM[1] : $AMPM[0] }
+
+sub d { sprintf("%02d",$_[0]->[3]) }
+sub e { sprintf("%2d",$_[0]->[3]) }
+sub H { sprintf("%02d",$_[0]->[2]) }
+sub I { sprintf("%02d",$_[0]->[2] % 12 || 12)}
+sub j { sprintf("%03d",$_[0]->[7] + 1) }
+sub k { sprintf("%2d",$_[0]->[2]) }
+sub l { sprintf("%2d",$_[0]->[2] % 12 || 12)}
+sub m { sprintf("%02d",$_[0]->[4] + 1) }
+sub M { sprintf("%02d",$_[0]->[1]) }
+sub s { 
+   $epoch = timegm(@{$_[0]}->[0..5])
+	unless defined $epoch;
+   sprintf("%d",$epoch) 
+}
+sub S { sprintf("%02d",$_[0]->[0]) }
+sub U { wkyr(0, $_[0]->[6], $_[0]->[7]) }
+sub w { $_[0]->[6] }
+sub W { wkyr(1, $_[0]->[6], $_[0]->[7]) }
+sub y { sprintf("%02d",$_[0]->[5] % 100) }
+sub Y { sprintf("%04d",$_[0]->[5] + 1900) }
+sub Z { defined $tzname ? $tzname : uc tz_name(undef, $_[0]->[8]); }
+
+sub z {
+ my $o = defined $tzname ? tz_offset($tzname) : tz_offset();
+ sprintf("%+03d%02d", int($o / 3600), abs(int($o % 3600)));
+}
+
+sub c { &x . " " . &X }
+sub D { &m . "/" . &d . "/" . &y  }      
+sub r { &I . ":" . &M . ":" . &S . " " . &p  }   
+sub R { &H . ":" . &M }
+sub T { &H . ":" . &M . ":" . &S }
+sub t { "\t" }
+sub n { "\n" }
+sub o { sprintf("%2d%s",$_[0]->[3],$Dsuf[$_[0]->[3]]) }
+sub x { my $f = $format{'x'}; _subs(shift,$f); }
+sub X { my $f = $format{'X'}; _subs(shift,$f); }
+sub C { my $f = $format{'C'}; _subs(shift,$f); }
+
+sub AUTOLOAD
 {
- my($fmt,$time);
+ use vars qw($AUTOLOAD);
 
- ($fmt,$time,$tzname) = @_;
-
- $epoch = $time;
-
- ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = @$time;
-
- if(defined $tzname)
+ if($AUTOLOAD =~ /::(\w)\Z/)
   {
-   $tzname = uc $tzname;
+   no strict qw(refs);
 
-   $tzname = sprintf("%+05d",$tzname)
-	unless($tzname =~ /\D/);
-
-   my $time = timegm($sec,$min,$hour,$mday,$mon,$year) +
-			tz_offset($tzname) - tz_offset();
-
-   ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = gmtime($time);
+   my $x = $1;
+   *{$x} = sub { $x };
+   goto &{$x};
   }
 
- $fmt =~ s#%([%a-zA-Z])#expand($1)#sge;
-
- $fmt;
+ die "$AUTOLOAD not defined in Date::Language::Generic";
 }
 
-sub ctime ($;$)
-{
- my($t,$tz) = @_;
- time2str("%a %b %e %T %Y\n", $t, $tz); 
-}
-
-sub asctime (\@;$)
-{
- my($t,$tz) = @_;
- strftime("%a %b %e %T %Y\n", @$t, $tz); 
-}
-
-
 1;
-
